$!*************************************************************************
$!*                                                                       *
$!*  JUMP -- BUILD_JUMP.COM                                               *
$!*                                                                       *
$!*************************************************************************
$!*  Author and Maintainer: Jonathan Ridler.                              *
$!*                                                                       *
$!*  This software is owned and maintained privately by the author,       *
$!*  Jonathan Ridler. It is NOT in any way produced, owned, supported,    *
$!*  maintained or endorsed by Hewlett Packard Pty Ltd or any of its      *
$!*  legal entities.                                                      *
$!*                                                                       *
$!*  Email: vmsjump@internode.on.net                                      *
$!*************************************************************************
$!*************************************************************************
$!***                                                                   ***
$!***                         LICENCE NOTICE                            ***
$!***                                                                   ***
$!***  This software is COPYRIGHT (c) 1993-2022 Jonathan Ridler.        ***
$!***  ALL RIGHTS RESERVED.                                             ***
$!***                                                                   ***
$!***  Please READ the file JUMP_LICENCE.TXT which contains the         ***
$!***  complete Licence and all conditions of use for JUMP.             ***
$!***                                                                   ***
$!***  NOTE: By using JUMP or any part of the JUMP distribution for	***
$!***        any purpose, you are agreeing to all conditions of the	***
$!***        JUMP licence.						***
$!***                                                                   ***
$!*************************************************************************
$!*************************************************************************
$!
$!  Build JUMP.
$!  Supports all architectures: VAX, Alpha, IA64/Integrity, X86_64.
$!
$!  Parameter values may be specified to control the build process.
$!  All parameters are optional. Some parameters must be the only parameter
$!  specified, but some parameters can be specifid together. When
$!  specifying more than one parameter, use individual separate parameters,
$!  i.e. P1 P2 etc. For example, to compile and link with traceback use
$!  this command:  @BUILD_JUMP C T
$!
$!    None    - Default action with no parameters: link with NO TRACEBACK
$!    B [xxx] - Create a backup saveset of all JUMP files; "xxx" is an
$!              optional string of arbitrary length representing the
$!              version ID; when specifying this option, it must be the
$!              ONLY option
$!    C       - Compile relevant Pascal sources and link with or without
$!              TRACEBACK as per the "L" (default) and "T" options
$!    D [xxx] - Create a JUMP distribution kit: compile ALL possible Pascal
$!              sources for the current architecture and link with
$!              NO TRACEBACK; "xxx" is an optional string of arbitrary
$!              length representing the version ID; when specifying this
$!              option, it must be the ONLY option
$!    L       - Link with NO TRACEBACK (default option)
$!    M file  - Compile a particular Pascal source file; a source filename
$!              is required; when specifying this option, it must be the
$!              ONLY option
$!    T       - Link with TRACEBACK; this option overrides the "L" option
$!    X       - Build X86_64 version with Cross-Compiler on IA64.
$!    Z       - Link with DEBUG; for Developer use only.
$!
$!-------------------------------------------------------------------------
$!  With new releases of OpenVMS, JUMP *MUST* be re-linked.
$!
$!  History:
$!      25-Sep-2001     JER     Updated for JUMP V4.0.
$!      07-May-2002     JER     Allow distribution builds.
$!      01-Apr-2004     JER     Allow build under V7.3-2+.
$!                              Also, display JUMP version at build time.
$!      27-May-2004     JER     Allow IA64 build.
$!      09-Oct-2011     JER     Add new options & streamline the builds.
$!      01-Feb-2012     JER     Add 'B' option; change parameter handling;
$!                              check Pascal compiler version and licence
$!                              status; add 'X' option; restructure code.
$!                              Not for VAX yet.
$!	03-Feb-2012	JER	Added VAX support.
$!	21-Feb-2012	JER	Minor cosmetic change.
$!	25-Nov-2022	JER	Add X86_64 support, changed X/Z options.
$!*************************************************************************
$
$ on ERROR  then  EXIT
$ say    := write SYS$OUTPUT
$
$ digits = "0123456789"
$
$ cross   = 0  ! Manually flag Use X86 cross-compiler
$ arch    = f$getsyi ("ARCH_NAME")                      ! VAX, Alpha, IA64, X86_64
$ version = f$edit (f$getsyi ("VERSION"),"TRIM")        ! OpenVMS version
$ VMS     = f$extract (1,5,version)                     ! = "X.X" or "X.X-X"
$ pid     = f$getjpi ("","PID")
$ trace  := /notraceback
$ source  = ""
$ valid_options = "BCDLMTXZ"
$
$ production := true                   ! Production or Development?
$ debug  = 0
$ dbg_c := /nodebug
$ dbg_l := /nodebug
$ dbg_m := /nodebug
$ dbg_n  = ""
$
$ VAX    = arch .eqs. "VAX"
$ Alpha  = arch .eqs. "Alpha"
$ IA64   = arch .eqs. "IA64"
$ X86_64 = arch .eqs. "x86_64"
$
$ if .not. production
$ then
$   say ""
$   say "*******************************************************************"
$   say "***           This is a DEVELOPMENT version of JUMP             ***"
$   say "*** Do NOT use this version of JUMP in production environments! ***"
$   say "*******************************************************************"
$   say ""
$ endif
$!
$!---------------------------------------------------------------------
$
$ compile = 0
$ linking = p1 .eqs. ""         ! Default action when no parameters
$ poker   = 0
$ single_compile = 0
$ licence = 0
$
$ cnt = 1
$ gosub CHECK_PARAMETER
$
$ save = p1 .eqs. "B"
$ dist = p1 .eqs. "D"
$ if dist .or. save
$ then
$   p2 = f$edit (p2,"UPCASE,UNCOMMENT,COLLAPSE")
$   if save  then  fyl = "JUMP''p2'_BACKUP.BCK"
$   if dist  then  fyl = "JUMP''p2'_DIST_KIT.BCK"
$   if f$parse (fyl,"SYNTAX_ONLY") .eqs. ""
$   then
$     say "%JUMP-F-BADFILNAM, Specified name for saveset is not valid"
$     EXIT
$   endif
$   cnt = 9
$ endif
$
$CHECK_PARAM:
$ if p1 .eqs. "M"
$ then
$   source = f$edit (p2,"UPCASE,UNCOMMENT,COLLAPSE")
$   if source .eqs. ""
$   then
$     say "%JUMP-F-NOSOURCE, ""M"" option requires a source file to be specified"
$     EXIT
$   else
$     if f$locate (".PAS",source) .eq. f$length (source)  then -
         source = source + ".PAS"
$     if f$search (source) .eqs. ""
$     then
$       say "%JUMP-F-NOFILE, Specified file does not exist"
$       EXIT
$     else
$       source = source - ".PAS"
$     endif
$   endif
$   single_compile = 1
$   cnt = 9
$ else
$   if .not. (save .or. dist)
$   then
$     cnt = cnt + 1
$     if p'cnt' .nes. ""  then  gosub CHECK_PARAMETER
$   endif
$ endif
$ if cnt .lt. 8  then  goto  CHECK_PARAM
$
$ if cross
$ then
$   if (f$logical("X86_XTOOLS$ROOT") .eqs. "")
$   then
$     write SYS$OUTPUT "%JUMP-F-NOX86ENV, X86 Cross Compiler environment NOT available!"
$     EXIT
$   endif
$   arch = "x86_64"
$   X86_64 := true
$   IA64   := false
$   version = "9.2"
$   define/job SYS$LIBRARY X86_XTOOLS$ROOT:[SYSLIB] ! ,SYS$SYSROOT:[SYSLIB]
$   define/job SYS$LOADABLE_IMAGES X86_XTOOLS$ROOT:[SYS$LDR]
$ else
$   if (f$logical("SYS$LIBRARY") .eqs. "X86_XTOOLS$ROOT:[SYSLIB]") then deassign/job SYS$LIBRARY
$   if (f$logical("SYS$LOADABLE_IMAGES") .eqs. "X86_XTOOLS$ROOT:[SYS$LDR]") then deassign/job SYS$LOADABLE_IMAGES
$ endif
$
$ linking = linking .or. dist .or. compile
$ link_status = 0
$
$ if VAX                                ! VAX architecture
$ then
$   nowarning = ""
$   poker = 1                           ! PSB not on VAX
$ endif
$
$ if Alpha                              ! Alpha architecture
$ then
$   nowarning = "/warning=noinformation"
$ endif
$
$ if IA64                               ! IA64/Itanium/Integrity architecture
$ then
$   nowarning = "/warning=noinformation"
$ endif
$
$ if X86_64                             ! X86_64 architecture
$ then
$   nowarning = "/warning=noinformation"
$ endif
$
$! If needs Pascal compiler, assume the licence is OK.
$ if (compile .or. single_compile .or. dist)    ! Need the Pascal compiler
$ then
$   if (f$search (f$parse("PASCAL","SYS$SYSTEM:.EXE")) .nes. "")
$   then
$     gosub GET_COMPILER_VERSION
$   else
$     say "%JUMP-F-NOPASCAL, Pascal is not installed or has no licence"
$     EXIT 1
$   endif
$ else                                          ! Not compiling
$   if linking .and. f$search ("JUMP.OBJ_''arch'''dbg_n'") .eqs. ""
$   then
$     say "%JUMP-F-NOOBJECT, JUMP Main object file does not exist"
$     EXIT 1
$   endif
$ endif
$
$ if (VMS .lts. "7.2") .or. poker       ! Force POKER, not PERSONA
$ then
$   changer := jump_persona_non_grata
$ else
$   changer := jump_persona
$ endif
$
$ gosub GET_JUMP_VERSION
$
$ if single_compile
$ then
$   say "Compiling ''source' ..."
$   pascal 'source' /usage=('usage_types') /object='source'.obj_'arch'
$ endif
$
$ if save
$ then
$   say "Creating backup of JUMP ''jump_version'for OpenVMS ''arch' ''version' ..."
$ else
$   if .not. single_compile
$   then
$     say "Building JUMP ''jump_version'for OpenVMS ''arch' ''version' ..."
$     if debug  then  say "***** DEBUG VERSION *****"
$   endif
$ endif
$
$!  Not all sites will have a Pascal compiler, so only compile if needed.
$
$ if dist .or. compile
$ then
$   if dist  then  say "Building JUMP distribution kit ..."
$
$   say "Compiling JUMP Pascal sources ..."
$   say "  ... Main program ..."
$   pascal jump /usage=('usage_types') /object=jump.obj_'arch''dbg_n' 'dbg_c'
$
$   if (.not. VAX) .and. (dist .or. (changer .eqs. "JUMP_PERSONA"))
$   then
$     say "  ... PERSONA module ..."
$     pascal jump_persona /usage=('usage_types') -
             /object=jump_persona.obj_'arch''dbg_n' 'dbg_c'
$   endif
$
$   if dist .or. (changer .eqs. "JUMP_PERSONA_NON_GRATA")
$   then
$     say "  ... PERSONA_NON_GRATA module ..."
$     pascal jump_persona_non_grata /usage=('usage_types') -
             /object=jump_persona_non_grata.obj_'arch''dbg_n' 'dbg_c'
$   endif
$
$   say "  ... POKER module ..."
$   pascal jump_poker /usage=('usage_types') -
           /object=jump_poker.obj_'arch''dbg_n' 'dbg_c'
$
$   say "  ... FIDDLER module ..."
$   pascal jump_fiddler /usage=('usage_types') -
           /object=jump_fiddler.obj_'arch''dbg_n' 'dbg_c'
$
$   if VAX
$   then
$     say "  ... ON_VAX module ..."
$     pascal jump_ON_VAX /usage=('usage_types') -
             /object=jump_ON_VAX.obj_vax'dbg_n' 'dbg_c'
$   else
$     say "  ... NOT_ON_VAX module ..."
$     pascal jump_NOT_ON_VAX /usage=('usage_types') -
             /object=jump_NOT_ON_VAX.obj_'arch''dbg_n' 'dbg_c'
$   endif
$ endif
$
$ if .not. (save .or. single_compile)
$ then
$
$!  Macro, CLD and Message compilers are standard on OpenVMS, so compile
$!  each time to ensure the object is correct for the architecture.
$
$   arch_defs = ""
$   if f$search ("SYS$LIBRARY:ARCH_DEFS.MAR") .nes. ""  then -
       arch_defs = "SYS$LIBRARY:ARCH_DEFS+SYS$DISK:[]"
$
$   say "Compiling JUMP Macro source ..."
$   macro 'arch_defs'jump_parse_id 'nowarning' -
          /object=jump_parse_id.obj'dbg_n' 'dbg_m'
$
$   say "Compiling JUMP CLD source ..."
$   set command /object jump_cld
$
$   say "Compiling JUMP Message source ..."
$   message jump_msg
$ endif
$
$ if linking
$ then
$   if trace .eqs. "/TRACEBACK"
$   then
$     say "Linking JUMP (with TRACEBACK) ..."
$   else
$     say "Linking JUMP ..."
$   endif
$
$   common = "jump.obj_''arch'''dbg_n'," + -
             "jump_fiddler.obj_''arch'''dbg_n'," + -
             "jump_poker.obj_''arch'''dbg_n'," + -
             "''changer'.obj_''arch'''dbg_n'," + -
             "jump_parse_id.obj''dbg_n',jump_cld.obj," + -
             "jump_msg.obj,jump.opt/option"
$
$   if VAX
$   then
$     link 'trace' 'dbg_l' 'common',            -
                jump_on_vax.obj_vax'dbg_n',     -
                SYS$SYSTEM:SYS.STB/select,      -
                SYSDEF.STB/select               -
                /executable=JUMP.EXE_VAX'dbg_n'
$     link_status = $status
$   endif
$
$   if Alpha
$   then
$     link /sysexe 'trace' 'dbg_l' 'common',            -
                jump_not_on_vax.obj_alpha'dbg_n',       -
                SYS$LOADABLE_IMAGES:SYSDEF.STB/select   -
                 /executable=JUMP.EXE_ALPHA'dbg_n'
$     link_status = $status
$   endif
$
$   if IA64
$   then
$     link /sysexe 'trace' 'dbg_l' 'common',            -
                jump_not_on_vax.obj_ia64'dbg_n',        -
                SYS$LOADABLE_IMAGES:SYSDEF.STB/select   -
                 /executable=JUMP.EXE_IA64'dbg_n'
$     link_status = $status
$   endif
$
$   if X86_64
$   then
$     link /sysexe 'trace' 'dbg_l' 'common',            -
                jump_not_on_vax.obj_x86_64'dbg_n',      -
                SYS$LOADABLE_IMAGES:SYSDEF.STB/select   -
                 /executable=JUMP.EXE_X86_64'dbg_n'
$     link_status = $status
$   endif
$ endif
$
$ if linking .and. link_status
$ then
$   say "*** JUMP ''jump_version'built successfully ***"
$   if debug  then  say "***** DEBUG VERSION *****"
$   say "EXECUTABLE filename is ",f$edit("JUMP.EXE_''arch'''dbg_n'","UPCASE")
$   say "Remember to install JUMP according to the installation instructions!"
$ endif
$
$ if dist .or. save
$ then
$   if save  then  say "Creating backup saveset ''fyl' ..."
$   if dist  then  say "Building distribution kit ''fyl' ..."
$   backup /log *.txt;,*.pas;,*.cld;,*.msg;,*.mar;,*.com;, -
                *.hlp;,*.opt;,*.dat;,*.obj_*;,*.exe*; 'fyl' /save_set
$ endif
$
$ if cross
$ then
$   if (f$logical("SYS$LIBRARY") .eqs. "X86_XTOOLS$ROOT:[SYSLIB]") then deassign/job SYS$LIBRARY
$   if (f$logical("SYS$LOADABLE_IMAGES") .eqs. "X86_XTOOLS$ROOT:[SYS$LDR]") then deassign/job SYS$LOADABLE_IMAGES
$ endif
$
$ EXIT 1
$!----------------------------------------------------------------------------
$!
$! Subroutines
$!
$!............................................................................
$!---  Check the supplied parameters and set options accordingly ...
$
$CHECK_PARAMETER:
$
$ px = f$edit (p'cnt',"UPCASE,UNCOMMENT,COLLAPSE")
$ if f$locate (px,valid_options) .eq. f$length (valid_options)
$ then
$   say "%JUMP-F-INVPARAM, Invalid parameter (""",px,""")"
$   STOP
$ endif
$
$ compile = compile .or. (px .eqs. "C")
$ linking = linking .or. (px .eqs. "L") .or. (px .eqs. "T")
$ if px .eqs. "T"  then  trace := /traceback
$ poker = poker .or. (px .eqs. "N")
$ debug = debug .or. (px .eqs. "Z")
$ cross = cross .or. (px .eqs. "X")
$
$ if debug
$ then
$   compile = 1                         ! Force re-compile
$   dbg_c := /debug=all/nooptimize
$   dbg_m  = dbg_c                      ! Alpha and IA64 only
$   if VAX  then  dbg_m := /debug=all   ! Different for VAX
$   dbg_l := /debug
$   dbg_n := _debug
$ endif
$
$ if (cnt .gt. 1) .and. -
     ((px .eqs. "B") .or. (px .eqs. "D") .or. (px .eqs. "M"))
$ then
$   say "%JUMP-F-ONEOPTION, ""''px'"" option must be the only option
$   EXIT
$ endif
$
$ RETURN
$
$!............................................................................
$!---  Analyze the compiler to get the version ID ...
$
$GET_COMPILER_VERSION:
$
$   temp1 := SYS$SCRATCH:JUMP_TEMP_FILE_'pid'1.TMP
$   temp2 := SYS$SCRATCH:JUMP_TEMP_FILE_'pid'2.TMP
$   set message /nofacility /noidentification /noseverity /notext
$   analyze /image 'f$parse("PASCAL","SYS$SYSTEM:.EXE")' /output='temp1'
$   set message /facility /identification /severity /text
$   search 'temp1' "image file identification" /output='temp2'
$   close fyl /nolog
$   open fyl 'temp2' /read /error=FYL_ERROR1 /end=FYL_ERROR1
$   read fyl buf /error=FYL_ERROR1 /end=FYL_ERROR1
$   buf = f$edit (buf,"UPCASE,UNCOMMENT,COMPRESS,TRIM") - -
          "IMAGE FILE IDENTIFICATION: """ - "PASCAL "
$   buf = f$extract (0,4,buf)
$FYL_ERROR1:
$   close fyl /nolog
$   if f$search (temp1) .nes. ""  then  delete /nolog 'temp1';*
$   if f$search (temp2) .nes. ""  then  delete /nolog 'temp2';*
$   usage_types = "all"                 ! Assume Pascal <= v5.2
$   if (buf .gts. "V5.2")  then -
        usage_types = usage_types + ",noperformance"
$
$ RETURN
$
$!............................................................................
$!---  Use the JUMP.OPT file to get the JUMP version ID ...
$
$GET_JUMP_VERSION:
$
$ jump_version = ""
$ temp := SYS$SCRATCH:JUMP_TEMP_FILE_'pid'.TMP
$ search jump.opt identification /output='temp'
$ close fyl /nolog
$ open fyl 'temp' /read /error=FYL_ERROR2 /end=FYL_ERROR2
$ read fyl jump_version /error=FYL_ERROR2 /end=FYL_ERROR2
$ jump_version = jump_version - "IDENTIFICATION = """ - """"
$ if f$locate (f$extract (1,1,jump_version),digits) .ne. 1 then -
	jump_version = "V" + jump_version
$ jump_version = "(" + jump_version + ") "
$FYL_ERROR2:
$ close fyl /nolog
$ if f$search (temp) .nes. ""  then  delete /nolog 'temp';*
$
$ RETURN
$
$!----------------------------------------------------------------------------
